package main

import (
	"net/http"
	redismgr "server/db/redis"
	"server/msg"
	"server/user"

	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"github.com/name5566/leaf/log"
)

func ParseProtoBuf(c *gin.Context) {
	var req msg.ReqLogin
	if err := c.ShouldBindBodyWith(&req, binding.ProtoBuf); err != nil {
		c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{
			"error":   "Invalid protobuf data",
			"details": err.Error(),
		})
		return
	}
	c.Set("protobuf_data", &req)
	c.Next()
}

func main() {
	r := gin.Default()

	// r.GET("/", func(c *gin.Context) {
	// 	c.String(200, "Hello, Gin!")
	// })

	// r.GET("/json", func(c *gin.Context) {
	// 	c.JSON(200, gin.H{"message": "Hello, JSON!"})
	// })

	// r.GET("/test", func(c *gin.Context) {
	// 	c.Redirect(http.StatusMovedPermanently, "http://www.google.com/")
	// })

	r.POST("/ReqLogin", ParseProtoBuf, ReqLogin)
	r.POST("/ReqRegister", ParseProtoBuf, ReqRegister)
	r.Run(":8080") // 默认监听 :8080
}

func ReqRegister(c *gin.Context) {
	req := c.MustGet("protobuf_data").(*msg.ReqRegister)

	log.Debug(req.Account, req.Password)

	userId, _ := redismgr.GetUserIDFromRedisByAP(req.Account, req.Password)

	if userId != "" {
		c.ProtoBuf(http.StatusOK, &msg.ResRegister{
			Success: false,
			ErrMsg: &msg.MsgError{
				ErrorCode: 101,
				ErrorMsg:  " Account is exist！",
			},
		})
		return
	}

	userId, Success := redismgr.SetUserIDFromRedisByAP(req.Account, req.Password)
	if !Success {
		c.ProtoBuf(http.StatusOK, &msg.ResRegister{
			Success: false,
			ErrMsg: &msg.MsgError{
				ErrorCode: 102,
				ErrorMsg:  "server registerHandler error",
			},
		})
		return
	}

	redismgr.SaveUserInfoToRedis(&msg.UserInfo{
		UserId: userId,
		MCoin:  0,
		MHead:  req.MHead,
		Name:   req.NikeName,
	})
	c.ProtoBuf(http.StatusOK, &msg.ResRegister{
		Success: true,
		ErrMsg:  nil,
	})
}

func ReqLogin(c *gin.Context) {
	req := c.MustGet("protobuf_data").(*msg.ReqLogin)
	// 返回 protobuf 响应
	c.ProtoBuf(http.StatusOK, req)

	UserId, err := redismgr.GetUserIDFromRedisByAP(req.Account, req.Password)
	if err != nil {

		c.ProtoBuf(http.StatusOK, &msg.ResLogin{
			NikeName: "",
			UserId:   "",
			ErrMsg: &msg.MsgError{
				ErrorCode: 101,
				ErrorMsg:  "Password or Account is error！",
			},
		})
		return
	}
	// 获取用户信息 ,如果新用户则创建一个
	userData := user.GetUserInfoById(UserId)
	c.ProtoBuf(http.StatusOK, &msg.ResLogin{
		UserInfo: &msg.UserInfo{
			UserId: userData.UserId,
			MHead:  userData.MHead,
			MCoin:  userData.MCoin,
			Name:   userData.Name,
			RoomId: userData.RoomId,
		},
		ErrMsg: nil,
	})
}
